;***************************************************************************
;* A P P L I C A T I O N   N O T E   F O R   T H E   A V R   F A M I L Y
;* 
;* Number		:AVR000
;* File Name		:"1200def.inc"
;* Title		:Register/Bit Definitions for the AT90S1200
;* Date	 		:97.12.11
;* Version		:1.20
;* Support telephone	:+47 72 88 87 20 (ATMEL Norway)
;* Support fax		:+47 72 88 87 18 (ATMEL Norway)
;* Support E-Mail	:avr@atmel.com
;* Target MCU		:AT90S1200
;*
;* DESCRIPTION
;* When including this file in the assembly program file, all I/O register
;* names and I/O register bit names appearing in the data book can be used.
;* 
;* The Register names are represented by their hexadecimal addresses.
;* 
;* The Register Bit names are represented by their bit number (0-7).
;* 
;* Please observe the difference in using the bit names with instructions
;* such as "sbr"/"cbr" (set/clear bit in register) and "sbrs"/"sbrc" 
;* (skip if bit in register set/cleared). The following example illustrates
;* this:
;* 
;* in		r16,PORTB		;read PORTB latch
;* sbr	r16,(1<<PB6)+(1<<PB5)	;set PB6 and PB5 (use masks (00 - ffh), not bit# (0 to 7) )
;* out  	PORTB,r16		;output to PORTB
;*
;* in		r16,TIFR		;read the Timer Interrupt Flag Register
;* sbrc	r16,TOV0		;test the overflow flag (use bit#  (0 to 7) )
;* rjmp	TOV0_is_set	;jump if set
;* ...				;otherwise do something else
;***************************************************************************

;***** Specify Device
.device AT90S1200

;***** I/O Register Definitions
.equ	SREG	=$3f  ; status register  
.equ	GIMSK	=$3b  ; General interrupt mask register  GIMSK  
.equ	TIMSK	=$39  ; Timer interrupt mask register  TIMSK
.equ	TIFR	=$38  ; Timer interrupt flag register  TIFR
.equ	MCUCR	=$35  ; microcontroller control register 
.equ	TCCR0	=$33  ; timer/counter 0 control register 
.equ	TCNT0	=$32  ; timer/counter 0 current count
.equ	WDTCR	=$21  ; watchdog timer control register
.equ	EEAR	=$1e  ; EEPROM address register 
.equ	EEDR	=$1d  ; EEPROM data reg 
.equ	EECR	=$1c  ; EEPROM control reg
.equ	PORTB	=$18  ; Port B            
.equ	DDRB	=$17  ; data direction register port B
.equ	PINB	=$16  ; Port B input current logic values
.equ	PORTD	=$12
.equ	DDRD	=$11  ; data direction register port D
.equ	PIND	=$10  ; Port D input current logic values
.equ	ACSR	=$08  ; analog compare status register

;***** Bit Definitions

.equ	INT0	=6    ; external interrupt 0 enable   =   bit 6 General interrupt mask register  GIMSK 

.equ	TOIE0	=1    ;Timer 0 Overflow interrupt enable  = bit 1 Timer interrupt mask register  TIMSK

.equ	TOV0	=1    ; timer 0 Overflow flag  = bit 1  Timer interrupt flag register  TIFR

.equ	SE		=5	; sleep enable   -- bit 5 MCUCR            
.equ	SM		=4	; sleep mode    -- bit 4 MCUCR  ( can be used as a general-purpose flag if sleep mode is not used)      
.equ	ISC01	=1	; Ext Int 0 sense control   -- bit 1 MCUCR
.equ	ISC00	=0	; Ext Int 0 sense control   -- bit 0 MCUCR
 
; timer 0 control register  TCCR0
.equ	CS02	=2
.equ	CS01	=1
.equ	CS00	=0
 
; watchdog timer control register WDTCR
.equ	WDE	=3
.equ	WDP2	=2
.equ	WDP1	=1
.equ	WDP0	=0

.equ	EEWE	=1
.equ	EERE	=0

.equ	PB7	=7
.equ	PB6	=6
.equ	PB5	=5
.equ	PB4	=4
.equ	PB3	=3
.equ	PB2	=2
.equ	PB1	=1
.equ	PB0	=0

.equ	DDB7	=7
.equ	DDB6	=6
.equ	DDB5	=5
.equ	DDB4	=4
.equ	DDB3	=3
.equ	DDB2	=2
.equ	DDB1	=1
.equ	DDB0	=0

.equ	PINB7	=7
.equ	PINB6	=6
.equ	PINB5	=5
.equ	PINB4	=4
.equ	PINB3	=3
.equ	PINB2	=2
.equ	PINB1	=1
.equ	PINB0	=0

.equ	PD6	=6
.equ	PD5	=5
.equ	PD4	=4
.equ	PD3	=3
.equ	PD2	=2
.equ	PD1	=1
.equ	PD0	=0

.equ	DDD6	=6
.equ	DDD5	=5
.equ	DDD4	=4
.equ	DDD3	=3
.equ	DDD2	=2
.equ	DDD1	=1
.equ	DDD0	=0

.equ	PIND6	=6
.equ	PIND5	=5
.equ	PIND4	=4
.equ	PIND3	=3
.equ	PIND2	=2
.equ	PIND1	=1
.equ	PIND0	=0

.equ	ACD	=7
.equ	ACO	=5
.equ	ACI	=4
.equ	ACIE	=3
.equ	ACIS1	=1
.equ	ACIS0	=0

.equ	INT0addr=$001	;External Interrupt0 Vector Address
.equ	OVF0addr=$002	;Overflow0 Interrupt Vector Address
.equ	ACIaddr =$003	;Analog Comparator Interrupt Vector Address

.def	ZL	=r30  ; indirect register addressing
