;  ************************************************************************************
;  *    KBtnv201.asm  -  PC keyboard to MIDI converter 
;  *	Alan Probandt Portland Oregon USA
;  *
;  * 	Code designed for the Atmel AVR Tiny11/12  
;  *
;  *     version 2.01	July 2, 2004
;  *	 no MIDI in -- only read keypress and send MIDI out
;  *		this enables 4MHz crystal operation on the MIDI in pin     
;  *     
;				
;    				Atmel AVR Tiny11
;	  RESET/ 	PIN 1     | VCC  PIN 8  +5V
;	4MHz Xtal	PIN 2 PB3 | PB2  PIN 7 MIDI Out			(tiny12) SCK 
;	4MHz Xtal	PIN 3 PB4 | PB1  PIN 6 keyboard clock INT0	(tiny12) MISO
;	   	        PIN 4 Gnd | PB0  PIN 5 keyboard data 		(tiny12) MOSI
;
;    1  o   o  6       Looking into the MINI DIN6 connector on the PC keyboard cable
;   2  o     o  5     1 clock   2 Gnd   3 data   4 no connect    5 +5volts  6 no connect
;    3   o o  4
;
;  see the site -- http://www.beyondlogic.org/keyboard/keybrd.htm
;
;  *
;  *  [ Use Terminal font in text editor to format this file correctly ]
;  *
;  *
;  *	PC keyboard clock signal on AVR pin 6 (INT0)
;  *	PC keyboard data signal on AVR pin 5 (PB0)
;  *	MIDI output on pin 7 (PB2).  Needs to handle 3.6v/360 ohms = 10 milliAmps of current in 32 uSec pulses.
;  *
;  *	  Every keypress takes about 840 microseconds to transmit to the AVR.  After each
;  *	scancode transmission completes, the final received bit sets the T flag in the
;  * 	AVR's SREG.  Decoding the scancode takes about 25 to 50 clock cycles, depending 
;  *    on the individual scancode, about 12 microseconds.
;  *      Most keypresses trigger either a Note On/Off or a Program Change message.  With
;  *	the Running Status flag active, Note and Program messages take two MIDI bytes. 
;  *    Including code overhead, two byte MIDI transmissions take about 680 microseconds.
;  *    MIDI Note off messages are sent as 0x90 with a velocity level of zero.
;				
;    				Atmel AVR Tiny11
;	  RESET/ 	PIN 1     | VCC  PIN 8  +5V
;	4MHz Xtal	PIN 2 PB3 | PB2  PIN 7 MIDI Out				(tiny12) SCK 
;	4MHz Xtal	PIN 3 PB4 | PB1  PIN 6 keyboard clock INT0	(tiny12) MISO
;	   	        PIN 4 Gnd | PB0  PIN 5 keyboard data 		(tiny12) MOSI
 	
;    
;  ********************************************************************************************
;    How the program works
;
;The program maps keypresses from a PS2 keyboard to MIDI note values.  All the keys (but one; F7)
;have scancodes that are less than $80.  The scan table is 128 bytes in size (00-$7f). The scan
;code is indexed into the table and the value returned is either a note, a patch number digit
;(keypad), a patch change command (function key), the All-Notes-Off command (space), sustain
; (CapsLock), or octave change (up/down arrow).
;
;
;The code first checks if the MIDI note in 'DecodeValue' is already playing by checking for its MIDI 
;note number in each of the eight storage registers R8-R15.  If the note is already in one of the 
;storage registers, then the user has held the typewriter key down long enough to trigger the auto-repeat.  
;In this case, the repeated keypress is ignored. The note continues to play uninterrupted.  There doesn't 
;seem to be any way to turn off the auto-repeat of the PC keyboard.
;If the note number from 'DecodeValue' is not found in the storage register set, the set is checked to 
;see if there is space for a new note.  The code is set to allow eight notes to be played at once.  
;Each note number is stored in one of the eight registers.  If a register is not holding a note number 
;(a note which is currently sounding by the synth), then the register holds the value 0xFFh.  The code 
;searches the storage registers for the first one that holds 0xFFh.  If there is none, then eight notes 
;are sounding and the code exits.  If a register is found with 0xFFh, the value from 'DecodeValue' is 
;copied into that register and the note value is sent to synthesizer as a Note-On.
;When a key is released, its corresponding note value is found in the register set (r8-r15).  The register
;is set to 0xFFh and a MIDI note-on message with velocity set to zero is sent to the synth for that 
;note number.  The synth releases the note.
;This technique allows individual keys to play without affecting the other keys that are being held down.


;.nolist
.include "Tn11def.inc"
.list

.equ	Clock			= 4000000
.equ	MIDItxpin		= 2	 ; MIDI Transmit pin
.equ	MIDIinPin		= 4
.equ	MIDItxPORT		= PORTb 
.equ	NewMIDIin		= 7      
.equ	MIDIoutDone		= 3
.equ	TimerMIDI		=  (256-(clock/(31250*2)))+6 ; 128+6=86h for 32uS periods
.equ	INITIAL_BITCOUNT 	= 11   ;  used by Get_Scan
.equ	PARITY_BITCOUNT 	= 3    ;  used by Get_Scan
.equ	NOTEOFFSET		= 33  
.equ	SPACEBAR_NotesOFF	= $c0
.equ	REGISTER_AVAILABLE	= $FF
.equ	POSSIBLE_NOTES 		= 8
.equ	FIRST_STORAGE_REG 	= 8
.equ	SUSTAIN_ON		= 127
.equ	SUSTAIN_OFF		= 0
.equ	NOTE_ON_VELOCITY  	= 0x50
.equ	NOTE_OFF_VELOCITY  	= 0
.equ	NOTE_ON_MSG 		= $90
.equ	CHANGE_VOICE_MSG 	= $c0
.equ	MIDI_CONTROLLER_MSG 	= $b0 
.equ	MIDI_HOLD1		= $40
.equ	ALL_NOTES_OFF_BYTE 	= 123 ; $7b
.equ	ALL_NOTES_OFF_2nd 	= 0
.equ	CapsToggleValue		= 0b01000000
.equ  	KBPort			= Pinb ;KBport,KBdataPin
.equ	KBDataPin		= 0
.equ	FIRST_VXMap_REG		= 26
.equ	VXMapRegCount		= 3
.equ	SecondDigitFlag		= 7

.equ	PSEUDO_STACK 		= $65 ; imitate a real stack when using ICE200 for 1200 chip
.equ	PSEUDO_STACK_POINTER 	= $3d

.equ	KBD_ACK			= $0fa
.equ	T_flag 			= 6
.equ	Sign_flag		= 4
.equ	SM_flag			= 4 ; Sleep Mode (unused) - delay loop counter bit flag { 0 = long period is timed out}
.equ 	PushSwitch1  		= 4

; keyflag register's bit values
.equ	Edge_flag	= 7 ; use up or down edge while inputting char from keyboard
.equ	Caps_flag	= 6 ; Caps Lock pressed on -- no NoteOff transmitted
.equ	Alt_flag	= 5 ; left Alt key is being pressed
.equ	Break_flag	= 4	; the last scancode was an $f0 - the break code sent when a key is released
.equ	OnOff_flag	= 3	; set means turn on the note whose MIDI number was decoded from the most recent scancode
.equ	E0_flag		= 1	; the extended scancode $E0 was sent - use the E0table to decode scancode
.equ	Run_flag	= 0 ; Running Status - when set, Note On/off sends only the note number and velocity


;*********************************************************************************************
;  Register definitions
;
;  lower registers
; r0 is used for EEPROM addressing
.def	DecodeValue	= r0  	; PC keyboard char decoded to NoteOn value, ChangeVoice value, or AllNotesOff cmd
.def   	CurrentVoiceNumber = r1 ; selected voice on synth - used for incrementing and decrementing voice
.def	SustainValue	= r2    ; temporary register used to change HOLD1 message value
.def	NoteOffsetValue = r3    ; holds current octave offset value -value stored between keypresses
.def	KPvalue		= r4   	; used to decode keypad digit presses-value stored between keypresses   
.def	MIDInow		= r5
.def	NextOut		= r6	; next byte to be transmitted for current MIDI output message
.def	LastOut		= r7	; last byte to be transmitted for current MIDI output message
.def	TimerMask	= r7

; R8-R15 note-storage registers  note-currently-playing list

;  upper registers
.def	temp		= r16	; scratch register            
.def	irqtemp		= r17  	; interrupt's scratch register

.def	bitcount	= r18	; used by INT0 irq  [getscan]	
.def  	scancode	= r19	; storage of data bits received from PC keyboard 
.def	scanbits	= r20 	; shift register for holding PC keyboard data while being assembled
.def	keyflags	= r21     
.def	MIDIcontrol	= r22	; MIDI soft-UART control register
.equ	Out_Active	= 0
.equ	Out_Stop	= 1	; not used in this version
.equ	Out_Start	= 2     ; not used in this version
.equ	Out_Phase	= 3	; not used in this version

.def	MIDIOut		= r23            
.def	MIDICounter	= r24   
.def	KPtemp		= r29	; temporary register used to decode keypad digit presses only
.def	regcnt		= r29   ; temp counter used  in PlayNote, TurnOffNote, ChangeVoice, Mode One setup    
.def	CVtemp		= r29	; temp counter for ChangeVoice

 
.cseg
.org   $0000
		rjmp 	reset 		; Reset handler $000
		
		rjmp	get_scan	; $001  External interrupt 0 handler  
	

		
;******************************************************************************
;                                                        		
;	Timer Overflow Interrupt Code                                            		
;                                                    
;	The falling edge of the MIDIin start bit triggers PinChange interrupt (someday).    		
;******************************************************************************
		
.org  OVF0addr          	; $003  Timer 0 overflow handler  
;	Since the analog comparator isn't used, it's OK to put the timer IRQ 
; routine here and save two clock cycles of IRQ latency by using no RJMP instruction.  
;    11 cycles total per IRQ if no byte being sent
		
		
OVT0:		ldi	irqtemp, TimerMIDI 
        	out	TCNT0, irqtemp		; restore the timer count to 16 uS period
          
          	sbrs	MIDIcontrol, Out_Active	; no, check for processing an output byte.		
          	reti        ; no MIDI being processed, so leave timer interrupt
          	
          	sbrc	Out_Phase
          	rjmp	OutPhaseSet
		
          	tst	MIDIcounter
          	breq	tmp_MIDIdone
          	cpi	MIDIcounter, 10
          	breq	tmp_DoStart
          	cpi	MIDIcounter,1
		breq	tmp_DoStop
          	
tmp_Shift:      lsr	MIDIOut
        	brcc	tmp_OutA0
        	sbi	PortB, MIDItxPin
        	rjmp	tmp_DoOutBit1
tmp_OutA0:  	cbi	PortB, MIDItxPin
       
tmp_DoOutBit1:	dec	MIDIcounter
		reti     ; 24 cycles total IRQ mid-byte     	
          
          
tmp_DoStart:	cbi	PortB, MIDItxPin
		dec	MIDIcounter
		reti       ; 19 cycles total IRQ start bit
          
tmp_DoStop:	sbi	PortB, MIDItxPin
		dec	MIDIcounter
		reti      ; 21 cycles total IRQ stop bit    
          
tmp_MIDIdone:	
		clr	MIDIcontrol
		mov	MIDIout, NextOut 
		mov	NextOut, LastOut
		ser	irqtemp
		mov	LastOut, irqtemp
		cp	MIDIout, irqtemp
		breq	OutputMessageDone
		cbi	PortB, MIDItxPin ; create start bit
		ldi	MIDIcounter, 9
		sbr	MIDIcontrol, ( 1 << Out_Active )
	
OutputMessageDone:	reti		

OutPhaseSet:	cbr	MIDIcontrol, ( 1 << Out_Active )
		reti

;;====================================================         
                              
		
;********************************************************
;
;	Main Code
;
;********************************************************	                        
RESET:
		ldi	temp,PSEUDO_STACK ; imitate a real stack when using ICE200 for 1200 chip
		out	PSEUDO_STACK_POINTER,temp	;init Stack Pointer - does nothing on real 1200 chip
        	
; set-up ports for input/output and pull-up resistors
		ser	temp
		out	portb,temp	; port B pull-up resistors are all on
		clr	temp
		out	ddrb,temp  ; port b is all inputs  the DIP switches are here Pb0 to PB3
		ldi	temp,(1<<MIDItxpin)
		out	ddrb,temp  ; port b is all inputs  the DIP switches are Pb0 to PB3 - pushswitch is PB4
           	
;*******************************************************************************************
;
;   Get keypresses from standalone PC keyboard - convert to notes and send to MIDI port
;
;   Subroutines:	'decode_data'
;
;	Interrupt procedures:	external interrupt int0 -- 'getscan'
;
;   Most time spent in 'GetChar' which checks T flag for fully received keypress from PC keybrd
;   When T is set, 'decode_data' called to convert keypress to note or cmd.
;*******************************************************************************************
		clr	temp
		clr	zl
rst0:		st	z,temp
		inc	zl
		cpi	zl, 26
		brne	rst0
		ldi	temp,(1<<ISC01) ; setup INT0 interrupt on falling edge
		out	MCUCR, temp  	
		 
		ldi	temp,  (1<<INT0)
		out	GIMSK,temp      ;  enable external INT0
		
		ldi	temp, TimerMIDI
		out	TCNT0, temp
		ldi	temp, (1 << TOV0)
		out	TIMSK, temp
		ldi	temp, ( (0<<CS02) | (0<<CS01) | (1<<CS00) )
		out	TCCR0, temp  ; start timer interrupt

		rcall	ClearNoteTable
		
;  Reset places the values zero to two in the VxMap register set 
		ldi	zl, FIRST_VXMap_REG - 1  ; fill note storage area with REGISTER_AVAILABLE sentinels
		ldi	regcnt, VXMapRegCount ; r26 r27 r28
		clr	temp
rst2:		inc	zl
		st	z, temp
		inc	temp
		dec	regcnt
		brne	rst2  
           	
	   	clr	KPvalue	; used to decode keypad digit presses
		clr	keyflags ;  edge = bit 7:    shift = bit 1:   Break = bit 0
		clr	MIDIcontrol
		ldi	bitcount,INITIAL_BITCOUNT ; = 11
		clt	;   T flag used to indicate that a char from kybd is ready to transmit
		ldi	temp, NOTEOFFSET
		mov	NoteOffsetValue, temp
		clr	CurrentVoiceNumber
		sei		; switch on the general interrupt enabler
	       
;*****************************************************************************
;
;  GetChar:  Main Loop of the Program  99.9% of time spent here
;
;*****************************************************************************	
      
GetChar:	; first check if a new scancode has come from the keyboard
		in	temp, SREG 
		sbrc	temp,T_flag   ; if T flag (SREG bit 6) is set, then char from keyboard is ready to transmit
		rjmp	DoNewScanCode
		
		; if none of the above conditions are valid, then poll again until one condition is valid
		rjmp	GetChar		
					
DoNewScanCode:	
		clt	; clear T flag - prepare for next char
		rjmp	decode_data  ; convert the scancode from the keyboard into an ASCII character
								; send scancode and keyflags - return DecodeValue
BackFromDecodeData:
		tst	DecodeValue    ; if DecodeValue =0, then ignore scancode - it's a shift character or break scancode
		brne	Valid_key     
		rjmp	GetChar
;  **************************************************************************
;   Valid key  -   send the processed scan code to the correct code handler
;   first test for SPACE to turn all notes off
;   then check for a regular key Note On/Off
;   then check for keypad digit press
;   then check for function key
;  **************************************************************************
Valid_key:
		ldi	temp,SPACEBAR_NotesOFF  ; $c0 is decode value for All_Notes_Off
		cp	DecodeValue, temp
		breq	AllNotesOff
		ldi	temp, $40
		cp	DecodeValue, temp
		brlo	PlayNote  ; note offsets are the lowest values in the scan table
		ldi	temp, $80
		cp	DecodeValue, temp
		brlo	ChangeVoice	; any DecodeValue between $40 and $7f is a function key press
		rjmp	DoDigit   ; br if sign reg bit set - DecodeValue's table offset is greater than $7f
 	
PlayNote:  
		sbrc	keyflags,OnOff_flag   ; if OnOff flag is set, then char is a MIDI note number to be played
		rjmp	playMIDInote
		sbrs	keyflags,OnOff_flag  ; if OnOff flag is clear, then char is a playing MIDI note to be turned off
		rjmp	TurnOffMIDInote
		
; ********************************************************************
;   	Keypad digits   	Decode Value = 0x80 to 0x8f
;
; if a keypad digit is pressed then check if it is the first or second digit pressed. 
; the first digit will have KPValue register equal to zero. 2nd digit will have bit 7 set and 
; first digit press (the ten's column value in bits 3 to 0).  
; Code arrives here with digit value of keypad press (plus 0x80) in DecodeValue register.
DoDigit: 	mov	temp, DecodeValue
		andi	temp, 0b00001111 ; clear bit7 (keypad scan flag) and leave the digit value
		mov	DecodeValue, temp
		tst	KPvalue  ; is this the first or second received digit from the keypad?
		brmi	SecondDigit
		mov	temp, DecodeValue
		sbr	temp, (1<<SecondDigitFlag)  ; the highest bit acts as a flag for 1st or 2nd digit
		mov	DecodeValue, temp	
		mov	KPvalue, DecodeValue
 		rjmp	GetChar
		
SecondDigit:
		mov	KPtemp, KPvalue   ; received 2nd digit, so multiply 1st digit by 10 and add 2nd
		cbr	KPtemp, (1 << SecondDigitFlag)     ; remove 2nd digit flag (bit 7 of KPvalue acts as 2nd digit flag)
		mov	KPvalue, KPtemp	; KPvalue now holds the first digit value in bits 3-0
		lsl	KPvalue			; First digit multiply by ten 
		lsl	KPvalue			; by multiplying by 2 three times and adding original value twice
		lsl	KPvalue			; bit 7 to carry, bit 0 cleared
		add	KPvalue, KPtemp
		add	KPvalue, KPtemp
		add	KPvalue, DecodeValue	;  max value 99 decimal in KPvalue
		mov	CurrentVoiceNumber, KPvalue
		clr	KPvalue      	; initialize for next keypad digit set
		rcall	DoProgramChange
		rjmp	GetChar                     
 		

DoNextMIDIOutput:
		ser	temp
		cpse	NextOut, temp
		rjmp	DNMO1
		rjmp	GetChar
DNMO1:		mov	MIDIout, NextOut
		mov	NextOut, LastOut
		mov	LastOut, temp
		ldi	MIDIcounter, 10                  
		sbr	MIDIcontrol, ( 1 << Out_Active ) ; enable MIDI out message transmission
		rjmp	GetChar	

; *********************************************************************************************
;  AllNotesOff 	1) 	Put Note_Available sentinal into the register bank for all possible notes.
;			All possible note spaces are available because all notes are off.
;		2)	Send the All_Notes_Off  MIDI control message to the synthesizer.
;		3)  jump back to GetChar routine
;	When the Hold1 is enabled (CapsLock key) then AllNotesOff doesn't work.
;*********************************************************************************************
AllNotesOff:  
		rcall	ClearNoteTable

ANO2:		cbr	keyflags, (1<<Run_flag) ; turn off Running status flag so next NoteOn will send 0x90 as 1st MIDI byte
		ldi	MIDIout, MIDI_CONTROLLER_MSG      ; send AllNotesOff message - 0xB0 0x7B 0x00
		ldi	temp, ALL_NOTES_OFF_BYTE
		mov	NextOut, temp
		ldi	temp, ALL_NOTES_OFF_2nd
		mov	LastOut, temp
		ldi	MIDIcounter, 10                  
		sbr	MIDIcontrol, ( 1 << Out_Active ) 	; enable new MIDI output	
  		
  		rjmp	GetChar

	           	
; **********************************************************************************************************
;  Function key was pressed - Decode Value has the Fn key number - 1
;  Reset places the values zero to seven in the VxMap register set.
;
;  If a function key is pressed, check if the left Alt key is being held down, (the Alt_flag will be set).
;  	If the Alt key is down, check if the function key is Fn1 to Fn3.
;  	  Fn1 to Fn3 yes, then put the current voice number into the selected function key's voice map register.
;	  Fn1 to Fn3 no, then send a Program Change Message 3 through 11 to the synth (depending on the Fn key).
;	If the Alt key is not being pressed, get the value in the selected function key's voice map register
;	  and send to the synth as a Program Change Message value.
;	 
;  
;  ChangeVoice 	1) 	Put Note_Available sentinel into the register bank for all possible notes.
;		This indicates that all possible note spaces are available because all notes are off.
;		2)  Send the MIDI program change message to the synthesizer.  This will turn off all
;		sounding notes for the selected MIDI channel (channel 1 by default).
;		3)  Jump back to GetChar routine.
;***********************************************************************************************************	            	
ChangeVoice:
		mov 	temp,DecodeValue                                                                   
		andi	temp, $0f ;  remove the EEPROM table offset for function key decoding
				
		cpi	temp, VXMapRegCount   ; only few registers available for voice number mapping
		brlo	CV0
		
; function key Fn4 or greater pressed, just send voice number 4 to 12 to the synth
		mov	CurrentVoiceNumber,temp
		rjmp	DoProgramChange
		         
CV0:		sbrc	keyflags, Alt_flag ; check Alt key     
		rjmp	AltPressed
		
; Alt key not pressed (Alt_flag is clear), so get value from VxMap reg and send to synth as a Program Change
		ldi	ZL,FIRST_VXMap_REG	
		add	ZL, temp 			
		ld	temp, z
		mov	CurrentVoiceNumber,temp ; temp now has voice number found in VxMap register for Fn key pressed
		rjmp	DoProgramChange
		
AltPressed:	 
		ldi	ZL,FIRST_VXMap_REG  
		add	ZL, temp        	
		st	z, CurrentVoiceNumber
		rjmp	GetChar  ; don't change voice number if Alt key is pressed, just map Current Voice Num to Fn key
		  		
DoProgramChange:
		rcall	ClearNoteTable
		cbr	keyflags, (1<<Run_flag) ; turn off Running status flag so next NoteOn will send 0x90 as 1st MIDI byte
		ldi	MIDIOut,CHANGE_VOICE_MSG
		mov	NextOut,CurrentVoiceNumber
		ser	temp
		mov	LastOut, temp
		ldi	MIDIcounter, 10                  
		sbr	MIDIcontrol, ( 1 << Out_Active ) 	; enable new MIDI output message
		
		rjmp  	GetChar
	
;********************************************************************************                                          
; GET_SCAN:  ; signal handler for external interrupt int0 (AVR pin 6)                                                      
;   called by falling transition of the keyboard clock signal                                                                 
;   assemble the scancode from keyboard data signal- data valid when clock is low
;  22 cycles for mid-byte data; 20 start and parity bits; 25 for stop bit
;                                       
; (  use Terminal font to see correctly )                                                                                  
;clk---|_|--|_|--|_|--|_|--|_|--|_|--|_|--|_|--|_|---|_|---|_|----------   (---- = logic high +5v)                         
;data-\___/-----\___/-----\___/-----\___/-----\___/--------------------                                                    
;    11   10    9    8    7    6    5    4    3     2     1 bitcount on falledge before dec                                         
;   start    0    1    2    3    4    5    6    7     P   stop   data bit                                                  
;  Bitcnt 3 to 10 is data. start bit = 11, parity bit=bit 2;  stop bit = bit 1                                             
;*********************************************************************************                                         
get_scan:                                                    
		cpi 	bitcount,INITIAL_BITCOUNT 	; falling edge of the FIRST pulse from the keyboard -start bit so do nothing   
		breq	GSD0                                                                                                      
		cpi	bitcount,PARITY_BITCOUNT 	; = 3  ; test for parity bit and stop bit                                      
		brlo	GSD0  					; must use bitcount 3 for compare because branch tests only for lower          
		lsr	scanbits   					; shift data right one bit - data bit 7 gets 0                                 
		sbic	KBport,KBdataPin  			; set scancode bit if keyboard data bit is set                                  
		ori	scanbits,$80 				; if data from kbrd is 1, then set bit 7 only and let other bits unchanged    	                                                                                                   
GSD0:		dec 	bitcount                                                                                                   
		brne	GSD1					; All bits received?                                                           
		ldi	bitcount,INITIAL_BITCOUNT                                                                              
		mov	scancode, scanbits
		set								; set T flag to let main program know a new char is ready   
GSD1:  		in	irqtemp, GIFR			; clear any pending interrupt INT0
		sbr	irqtemp,INTF0
		out	GIFR, irqtemp 				; GIMSK and MCUCR set for interrupt on falling edge of INT0
		reti
	     	
;*****************************************************************************************************************
;  DECODE_DATA  -- routine to convert the scancode from the keyboard. 
;	Not a subroutine. DECODE_DATA's single exit point jumps back to Main code.
;   input: scancode (register value)   output = DecodeValue (register value)
;
;   A key pressed down will send the one-byte scancode to the PC for most characters.
;   When the key is released, the keyboard sends an $f0 byte followed by the scancode of the key being released.
;   This routine uses a flag called Break to determine if the scancode comes from a keypress down or release.
;	Break == clear means that scancode just received is a keypress down and should be decoded to Note On.
;	Break == set means that the scancode just received was $f0 and the next scancode means turn Note Off.
;
; 	Shift keys are ignored
;*****************************************************************************************************************
decode_data:
;   Check for the one key (Fn7) that is outside of the table range
		cpi	scancode, $83
		brne	InRange
		ldi	temp, $46
		mov	DecodeValue, temp
		rjmp	exit_decode
		
;   first check that Break flag is clear. if yes, then keypress down received
InRange:	sbrc	keyflags, Break_flag   ; the previous scancode was $f0 when break_flag is set => key released
		rjmp	Break_set

;   Break is clear, so do check the first three special conditions	
		cpi	scancode,$f0  ; 	$F0 =Breakcode  finger-off-the-key identifier
		brne	notF0
		sbr	keyflags, (1<<Break_flag)  ; set Break flag  so that the next scancode will not be seen a keypress down	
		rjmp	setZero
		
notF0:  	cpi	scancode, $11 ; left Alt key is being pressed down. Alt_flag gets set.  Next key should be Fn1 to Fn8.
		brne	isCapsLock
		sbr	keyflags, (1<<Alt_flag)
		rjmp	setZero
		
; CapsLock key pressed -- Code to turn off/on Sustain Petal			
isCapsLock:  
		cpi	scancode, $58  ; caps lock key
		brne	isExtendedCode
		ldi	temp, CapsToggleValue
		eor	keyflags, temp  ; toggle the value in keyflags reg for sustain
 		nop	; make sure the that the register gets updated value before reading it
		sbrc	keyflags, Caps_flag
		rjmp	TurnOnSustain  ; flag toggled to set so initialize note sustain
		clr	NextOut ; turn off sustain petal
		rjmp	Sustain
TurnOnSustain:	ldi	temp, SUSTAIN_ON
		mov	NextOut,temp
Sustain:	cbr	keyflags, (1<<Run_flag) ; turn off Running status flag so next NoteOn will send 0x90 as 1st MIDI byte
; set up MIDI message for sustain controller change
		ldi	MIDIOut,MIDI_CONTROLLER_MSG
 		ldi	temp, 0xff
		mov	LastOut, temp		
		ldi	MIDIcounter, 10
		sbr	MIDIcontrol, ( 1 << Out_Active ) 		
		rjmp 	setZero
		
isExtendedCode: ; check for E0 flag
		cpi	scancode,$e0  ;  $e0  extended char table
		brne	isE0set  ; scancode is not one of the special cases
		sbr	keyflags,(1<<E0_flag) ; set E0 flag			
		rjmp	setZero	
	
isE0set: ; if the E0_flag is set then E0 (extended char scancode) was the last previously sent
		sbrc	keyflags,E0_flag 	; test extended char flag 
		rjmp	get_E0char		; determine the action to take for the extended keypress    
 
;not shift, $E0, or $F0, and no flags are set;  so do a table lookup from the EEPROM section 
TableLookUp:
		ldi	ZL, low(2 * ScanTable) 
		ldi	ZH, high(2 * ScanTable)
		add	ZL, scancode
		brcc	TL0
		inc	ZH          
TL0:		lpm     	                          
		sbr	keyflags,(1<<OnOff_flag)  ; set the note on/off flag		
		rjmp 	exit_decode
	
;  Break flag is set - this means the previous scancode was $f0 (a key release)
;   This scancode is either the Extended sentinel ($e0) or the number of the key being 
;	released. If the scancode is in the EEPROM table, then turn OFF the MIDI note.
;   If the released key has an extended code, then the clear the break and the 
;	extended flags, set DecodeValue to zero and exit.
Break_set:
		cbr	keyflags,(1<<Break_flag) ; clear Break flag
		cpi	scancode, $11	; left Alt flag is released
		brne	isE0
		cbr	keyflags, (1<<Alt_flag)
		rjmp	setZero
		
; If scancode is E0, then set the Break flag so that the next scancode is recognized 
; as the last of an extended release sequence instead of a normal keypress down.		
isE0:		cpi	scancode, $e0	; extended key press sentinel
		brne	isE0flagOn
		sbr	keyflags,(1<<Break_flag)
		sbr	keyflags,(1<<E0_flag) ; set extended flag		
		rjmp	setZero
	
; if the E0flag is on then this scancode is the extended character number that is being released.  Just clear flags.		
isE0flagOn:		
		sbrs	keyflags, E0_flag   ; Extended flag on?
		rjmp	TableLookUp1		;no, a note key is being released.  Find note and send MIDI note off message.
		cbr	keyflags,(1<<Break_flag) ; yes, clear Break flag
		cbr	keyflags,(1<<E0_flag) ; clear extended flag
		rjmp	setZero	
		
TableLookUp1:      ;determine which note needs to be turned off
		ldi	ZL, low(2 * ScanTable) 
		ldi	ZH, high(2 * ScanTable)
		add	ZL, scancode
		brcc	TL1
		inc	ZH           
TL1:		lpm  
		mov	temp, DecodeValue
   	        cpi	temp, $40
   	        brsh	setZero
   	        
   	        	
		cbr	keyflags,(1<<OnOff_flag)  ; clear to tell main that note goes off       
		rjmp	exit_decode
setZero:  
		clr	DecodeValue	; When Decode sub returns zero in DecodeValue reg, the main program ignores the scancode.
exit_decode: 		; subroutine's single exit point
		rjmp	BackFromDecodeData

;  the previous scancode was E0, and few extended keys are handled by the AVR1200 code.
;  The ones that are include the up/down arrow keys for changing octave range and the
;  left/right arrow keys for changing to the previous/next voice number.
get_E0char:
		cbr	keyflags,(1<<E0_flag)  ; clear the E0 flag
		cpi 	scancode,$72  	; up arrow - raise the note value of all the keys by one octave.
		brne	isDownArrow
		mov	temp,NoteOffsetValue
		cpi	temp,$7f-12
		brsh	setZero 		; branch is same or higher (unsigned)
		subi	temp,-12		; 'addi' in AVR is done by subtracting a negative number 
		mov	NoteOffsetValue,temp
		rjmp	setZero
	
isDownArrow:
		cpi	scancode,$75	; down arrow - lower the note value of all the keys by one octave
		brne	isLeftArrow
		mov	temp,NoteOffsetValue; can't send a MIDI note on note that is less than zero
		cpi	temp,12  		; so check that NoteOffset value is at least 12 before subtracting 12
		brlo	setZero  		; branch is same or higher (unsigned)
		subi	temp,12			; lower the note value of all the keys by one octave
		mov	NoteOffsetValue,temp
		rjmp	setZero
		
isLeftArrow:
		cpi	scancode,$6b	; left arrow - change voice to previous number in synth voice bank
		brne	isRightArrow	
		mov	temp, CurrentVoiceNumber
		dec	temp
		andi	temp, 0b01111111
		mov	CurrentVoiceNumber,temp
		rjmp	DoProgramChange
		
isRightArrow:
		cpi	scancode,$74	; right arrow  - change voice to next number in synth voice bank
		brne	setZero			
		mov	temp, CurrentVoiceNumber
		inc	temp
		andi	temp, 0b01111111
		mov	CurrentVoiceNumber,temp
		rjmp	DoProgramChange	  ; do lots of jumping to minimize stack usage on AVR1200	
; ***** end of DecodeData subroutine *******************

;.include "KBtnAux.asm" ; split the code file here when using WAVRASM.exe Atmel Windows AVR assembler.
; For WAVRASM, put all the following code into the file "KBtnAux.asm", delete from this file,
;   and uncomment the above INCLUDE statement.  Truly a nitwit situation!  C'mon, Atmel, fix this!

;*******************************************************************
;  playMIDInote    not a subroutine
;   receives: keyflags, DecodeValue (holds note number without Octave Offset)
;*******************************************************************
playMIDInote:	add	DecodeValue, NoteOffsetValue
		ldi	ZL,FIRST_STORAGE_REG
		ldi	regcnt,POSSIBLE_NOTES
PMN1:		ld	temp,z
		cp	temp, DecodeValue
		breq	PMN_OutOfRegs ; found identical note number, which means that the Note had been already playing.
;					The key was held down long enough trigger typematic auto-repeat.  Do nothing and exit
		inc	zl  ; first check 8 storage registers for same DecodeValue to see if the note is already playing
		dec	regcnt
		breq	NewNote  ;DecodeValue not found in list of notes that are currently playing
		rjmp	PMN1
				
NewNote: 	; now check if there is space in the note-currently-playing list for a new note
		ldi	ZL,FIRST_STORAGE_REG
		ldi	regcnt,POSSIBLE_NOTES
PMN2:		ld	temp,z  ; load the value of the dereferenced slot in the currently-playing list
		cpi	temp,REGISTER_AVAILABLE  ; check if a note value is occupying this space in the currently-playing list
		breq	PMN3 ; selected slot filled with a playing note, so check the next slot
		inc	zl  
		dec	regcnt
		breq	PMN_OutOfRegs ; no space for the new note was found in the note-currently-playing list
		rjmp	PMN2
		
; store the value of the note offset plus the octave offset in CurrentNoteOn list		
PMN3:		mov	temp, DecodeValue
	;	add	temp, NoteOffsetValue 
		st	z,temp ; now put the DecodeValue (note value minus octave offset) into the currently-playing list 

  		sbrc	keyflags, Run_flag     ; check if previous MIDI message was a Note On
     		rjmp	PMN4  ; Running Status is on, so don't send Note-On MIDI command byte                   
     		sbr	keyflags, (1<<Run_flag)	; turn on the RunFlag
		ldi	MIDIOut,NOTE_ON_MSG
		mov	NextOut, temp
		ldi	temp,NOTE_ON_VELOCITY
		mov	LastOut,temp
		rjmp	PMN5
		

	; RunningStatus is on, so only send two bytes for MIDI note-on

PMN4:		mov	MIDIOut,temp
		ldi	temp,NOTE_ON_VELOCITY
		mov	NextOut,temp
		ser	temp
		mov	LastOut, temp
		
PMN5:		ldi	MIDIcounter, 10                  
		sbr	MIDIcontrol, ( 1 << Out_Active )  ; enable MIDI out message transmission		
	
PMN_OutOfRegs:	rjmp	GetChar

;*********************************************************
;. TurnOffMIDInote    not a subroutine
;   receives: keyflags, DecodeValue 
;   registers used:  keyflags, regcnt
TurnOffMIDInote:         
; look for note in the CurrentNotePlaying list table
		add	DecodeValue, NoteOffsetValue
		ldi	ZL,FIRST_STORAGE_REG  
		ldi	regcnt,POSSIBLE_NOTES
ToMN1:  	ld	temp,z	
		cp	temp,DecodeValue
		breq	ToMN2
		inc	zl		        	
		dec	regcnt
		breq	ToMN_OutOfRegs
			
		rjmp	ToMN1
	
ToMN2: ; found note number in register storage
		ldi	temp,REGISTER_AVAILABLE  ; make this slot in the currently-playing list available for a new note
		st	z,temp  ; let program that this reg is available 
  		sbrc	keyflags, Run_flag     ; check if Running Status is active (previous MIDI message was a Note On)
     		rjmp	ToMN3  ; Running Status is on, so don't send Note-On MIDI command byte                   
     		sbr	keyflags, (1<<Run_flag)	; Running Status is off, so set Run_flag and send Note-On MIDI command byte	
		ldi	MIDIout, NOTE_ON_MSG
		add	DecodeValue, NoteOffsetValue	
		mov	NextOut, DecodeValue
		ldi	Temp, NOTE_OFF_VELOCITY
		mov	LastOut, temp
		rjmp	toMN4
		
ToMN3:  	mov	MIDIOut, DecodeValue
		ldi	temp, NOTE_OFF_VELOCITY
		mov	NextOut, temp
		ser	temp
		mov	LastOut, temp
toMN4:		ldi	MIDIcounter, 10                  
		sbr	MIDIcontrol, ( 1 << Out_Active ) 	
		rjmp	getchar
		
ToMN_OutOfRegs:	rjmp	AllNotesOff  ; couldn't find note in storage so turn off all notes
	
ClearNoteTable:
		ldi	ZL, FIRST_STORAGE_REG  ; clear the note storage regs because changing voice turns off all notes
		ldi	regcnt, POSSIBLE_NOTES
		ser	temp
CNT1: 		st	z,temp
		inc	zl  ;a change voice command turns off all sounding notes
		dec	regcnt
		brne	CNT1 ; make all storage registers available for new notes when voice changes
		ret

ID:		
.db   "****  BKNT2v10a.ms lAnaP orabdn tuJ l3  ,0240* ****"  
;.db  "****  KBTNv201.asm Alan Probandt Jul  3, 2004 *****"

		
;;********************************************************************
;;   keypress table  
; scancode is offset into table
;  table value < $3f  keyboard note (octave offset added to get MIDI note value)
;  table value $40 - $4f   function key
;  table value $80 - $8f   keypad press
;  table value $c0	space (All Notes Off)
;  table value $f0 - $fe   special key
;  table value $ff   filler value ignore
;;*********************************************************************                                                          

ScanTable:
;    0   1   2   3   4   5   6   7   8   9   a   b   c   d   e   f
.db $ff,$48,$ff,$44,$42,$40,$41,$4b,$ff,$49,$47,$45,$43,$ff,  0,$ff ; 00-0f
.db $ff,$ff, 25,$ff,$f1,  2,  1,$ff,$ff,$ff, 26, 27, 24,  4,  3,$ff ; 10-1f
.db $ff, 30, 28, 29,  6,$ff,  5,$ff,$ff,$c0, 31,$ff,  9,  7,  8,$ff ; 20-2f
.db $ff, 35, 33, 34, 32, 11, 10,$ff,$ff,$ff, 37, 36, 13, 12,$ff,$ff ; 30-3f
.db $ff, 38,$ff, 14, 16, 17, 15,$ff,$ff, 40, 42, 39, 41, 18,$ff,$ff ; 40-4f
.db $ff,$ff, 44,$ff, 19, 20,$ff,$ff,$ff, 43, 45, 21,$ff, 23,$ff,$ff ; 50-5f
.db $ff,$ff,$ff,$ff,$ff,$ff, 22,$ff,$ff,$81,$ff,$84,$87,$ff,$ff,$ff ; 60-6f
.db $80,$8e,$82,$85,$86,$88,$f2,$8a,$4a,$8d,$83,$8c,$8b,$89,$f4,$ff ; 70-7f


                   