;***************************************************************************
;* A P P L I C A T I O N   N O T E   F O R   T H E   A V R   F A M I L Y
;* 
;* Number		:AVR000
;* File Name	:"tn11def.inc"
;* Title		:Register/Bit Definitions for the ATtiny11
;* Date	 		:03.07.16
;* Version		:1.00
;* Support telephone	:+47 72 88 43 88 (ATMEL Norway)
;* Support fax		:+47 72 88 43 99 (ATMEL Norway)
;* Support E-Mail	:avr@atmel.com
;* Target MCU		:ATtiny11
;*
;* DESCRIPTION
;* When including this file in the assembly program file, all I/O register
;* names and I/O register bit names appearing in the data book can be used.
;* 
;* The Register names are represented by their hexadecimal addresses.
;* 
;* The Register Bit names are represented by their bit number (0-7).
;* 
;* Please observe the difference in using the bit names with instructions
;* such as "sbr"/"cbr" (set/clear bit in register) and "sbrs"/"sbrc" 
;* (skip if bit in register set/cleared). The following example illustrates
;* this:
;* 
;* in	r16,PORTB		;read PORTB latch
;* sbr	r16,(1<<PB6)+(1<<PB5)	;set PB6 and PB5 (use masks, not bit#)
;* out  PORTB,r16		;output to PORTB
;*
;* in	r16,TIFR		;read the Timer Interrupt Flag Register
;* sbrc	r16,TOV0		;test the overflow flag (use bit#)
;* rjmp	TOV0_is_set		;jump if set
;* ...				;otherwise do something else
;***************************************************************************

;***** Specify Device
;.device ATtiny11

;***** I/O Register Definitions
.equ	SREG	=$3f	; status register
.equ	GIMSK	=$3b	; General interrupt mask register 
.equ	GIFR	=$3a	; General interrupt flag register
.equ	TIMSK	=$39	; Timer interrupt mask register
.equ	TIFR	=$38	; Timer interrupt flag register
.equ	MCUCR	=$35	; microcontroller control register
.equ	MCUSR	=$34	; microcontroller status register
.equ	TCCR0	=$33	; timer/counter 0 control register
.equ	TCNT0	=$32	; timer/counter 0 current count
; .equ	OSCCAL  =$31	; Oscillator Calibration Register  Tiny12 only
.equ	WDTCR	=$21	; watchdog control register
; .equ	EEAR	=$1e	; EEPROM Address register      Tiny12 only 
; .equ	EEDR	=$1d	; EEPROM Data register        Tiny12 only 
; .equ	EECR	=$1c	; EEPROM Control register Tiny12 only 
.equ	PORTB	=$18	; Port B output logic levels
.equ	DDRB	=$17	; Data Direction Port B
.equ	PINB	=$16	; Current Input Logic Port B
.equ	ACSR	=$08	; Analog Compare Status register


;***** Bit Definitions

;General interrupt mask register  GIMSK $3b 
.equ	INT0	=6		; external interrupt 0 enable  (pin 6)
.equ	PCIE1 	=5		; pin change interrupt enable

;General interrupt flag register    GIFR $3a
.equ	INTF0	=6		; external interrupt flag 0 (pin 6)
.equ	PCIF	=5		; pin change interrupt flag

;Timer interrupt mask register  TIMSK $39 
.equ	TOIE0	=1		; Timer 0 Overflow interrupt enable

; Timer interrupt flag register    TIFR $38
.equ	TOV0	=1		; Timer 0 Overflow flag

; microcontroller control register  MCUCR $35
.equ	SE		=5		; sleep mode enable
.equ	SM		=4		; sleep mode select
.equ	ISC01	=1		; interrupt sense control 0 bit 1
.equ	ISC00	=0		; interrupt sense control 0 bit 0

; microcontroller status register  MCUSR $34
.equ	EXTRF	=1 		; external reset flag
.equ	PORF	=0 		; power-on reset flag

; timer/counter control register zero TCCR0 $33
.equ	CS02	=2		; timer 0 clock select bit 2
.equ	CS01	=1		; timer 0 clock select bit 1
.equ	CS00	=0		; timer 0 clock select bit 0


; Watchdog control register WDTCR $21
.equ	WDTOE	=4		; watchdog change enable
.equ	WDE		=3		; watchdog enable
.equ	WDP2	=2		; watchdog timer prescaler bit 2
.equ	WDP1	=1		; watchdog timer prescaler bit 1
.equ	WDP0	=0		; watchdog timer prescaler bit 0
    	
; Port B output logic
.equ	PB4		=4
.equ	PB3		=3
.equ	PB2		=2
.equ	PB1		=1
.equ	PB0		=0

; Port B data direction register
.equ	DDB4	=4
.equ	DDB3	=3
.equ	DDB2	=2
.equ	DDB1	=1
.equ	DDB0	=0

; Port B input pin logic
.equ	PINB5	=5
.equ	PINB4	=4
.equ	PINB3	=3
.equ	PINB2	=2
.equ	PINB1	=1
.equ	PINB0	=0


; Analog Compare Control/Status register - ACSR $08
.equ	ACO 	=5		; analog compare output
.equ	ACI		=4		; analog compare interrupt flag
.equ	ACIE 	=3		; analog compare interrupt enable
.equ	ACIS1	=1		; Analog Compare interrupt mode select 1
.equ	ACIS0	=0		; Analog Compare interrupt mode select 0


.def	ZL		=r30
.def	ZH		=r31


.equ	INT0addr=$001	;External Interrupt0 Vector Address
.equ	PinCaddr=$002	;Pin Change Handler
.equ	OVF0addr=$003	;Overflow0 Interrupt Vector Address
.equ	ACIaddr =$004	;Analog Comparator Interrupt Vector Address


;; interrupt vector address jump table for ATTiny11
; cut-and-paste to your assembler source file, 
;  then remove the comment markers at the line beginnings.
;
;.org $0  ; interrupt vector address jump table for ATTiny11
;	rjmp	reset
;.org    INT0addr	;=$001
;   reti	;rjmp	EXT_INT0		
;.org    PinCaddr	;=$002
;	reti	;rjmp	PIN_CHANGE
;.org 	OVF0addr	;=$003  
;	reti	;rjmp	TIM0_OVF
;.org   ACIaddr   ;=$004
;	reti	;rjmp	ANA_COMP


;; CRYSTAL-DEPENDENT TIMING CONSTANTS

;;  3.6864MHZ  (0.271 microSec period)
;    XTAL/8  = 460800 (2.17 microSec period)
;    XTAL/64 = 57600 (17.36 microSec period)
;    XTAL/256 = 14400  (69.44 microSec period)
;    XTAL/1024 =3600  (277.7 microSec period)
; XTAL/9600 = 384   XTAL/8/9600 = 48  
; XTAL/38400 = 96   XTAL/8/38400 = 12 

;		Atmel AVR Tiny11 microcontroller                                                
;		      _______                                 
; 	reset/PB5|1 	 8| Vcc
;  	Xtal1/PB3|2 	 7| PB2 / t0
;  	Xtal2/PB4|3  	 6| PB1 / INT0/AIN1
; 		  Gnd|4  	 5| PB0 / AIN0

